import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.awt.Color;

import Jcg.geometry.*;
import Jcg.viewer.old.Fenetre;
import processing.core.PApplet;

/**
 * Methods for visualizing (colored and/or clustered) point clouds in 2D and 3D
 * 
 * @author Luca Castelli Aleardi and Steve Oudot(Ecole Polytechnique, INF562)
 * @version jan 2014
 */
public class Draw {

	/**
	 * Draw a point cloud in a 2D frame (using x and y coordinates)
	 * 
	 * @param points  the input points
	 */
    public static void draw2D(LinkedList<Point_2> points, String title) {
    	//System.out.println("Drawing point cloud in 2D... ");
    	if(points==null) {
    		System.out.println("No point to draw");
    		return;
    	}
    	
    	int dimension=points.getFirst().dimension();
    	if(dimension!=2) {
    		System.out.println("warning: wrong point dimension "+dimension);
    	}
    	
    	Fenetre f=new Fenetre(title);
    	f.addPoints(points);
    	
    	double[] box=PointSetUtil.boundingBox(points);
    	draw2DBoundingBox(f, box);
    }

	/**
	 * Draw a point cloud in a 2D frame (using x and y coordinates) within a given bounding box
	 * 
	 * @param points  the input points
	 */
    public static void draw2D(LinkedList<Point_2> points, double[] boundingBox, String title) {
    	//System.out.println("Drawing point cloud in 2D... ");
    	if(points==null) {
    		System.out.println("No point to draw");
    		return;
    	}
    	
    	int dimension=points.getFirst().dimension();
    	if(dimension!=2) {
    		System.out.println("warning: wrong point dimension "+dimension);
    	}
    	
    	Fenetre f=new Fenetre(title);
    	f.addPoints(points);
    	
    	draw2DBoundingBox(f, boundingBox);
    }

	/**
	 * Draw a point cloud in a 2D frame (using x and y coordinates)
	 * 
	 * @param points  the input points
	 */
    public static void draw2DTree(KdTree<Point_2> tree, String title) {
    	//System.out.println("Drawing point cloud in 2D... ");
    	if(tree==null || tree.root==null) {
    		System.out.println("Warning: the kd-tree is empty or not defined");
    		return;
    	}
    	
    	LinkedList<Point_2> points=tree.getPoints();
    	
    	Fenetre f=new Fenetre(title);
    	f.addPoints(points);
    	
    	double[] box=PointSetUtil.boundingBox(points);
    	draw2DBoundingBox(f, box);
    	
    	draw2DTreeRec(f, tree.root, box[0], box[1], box[2], box[3]);
    }

	/**
	 * Draw a point cloud in a 2D frame (using x and y coordinates)
	 * 
	 * @param points  the input points
	 */
    private static void draw2DTreeRec(Fenetre f, KdTreeNode<Point_2> tree, double xmin, double ymin, double xmax, double ymax) {
    	if(tree.lowerHalf==null && tree.upperHalf==null)
    		return;
    	
    	Point_2 p, q;
    	if(tree.cutDimension==0) {
    		p=new Point_2(tree.cutValue, ymin);
    		q=new Point_2(tree.cutValue, ymax);
    		draw2DTreeRec(f, tree.lowerHalf, xmin, ymin, tree.cutValue, ymax);
    		draw2DTreeRec(f, tree.upperHalf, tree.cutValue, ymin, xmax, ymax);
    	}
    	else {
    		p=new Point_2(xmin, tree.cutValue);
    		q=new Point_2(xmax, tree.cutValue);
    		draw2DTreeRec(f, tree.lowerHalf, xmin, ymin, xmax, tree.cutValue);
    		draw2DTreeRec(f, tree.upperHalf, xmin, tree.cutValue, xmax, ymax);
    	}
    	f.addSegment(p, q);
    }

	/**
	 * Draw a 2D bounding box (using x and y coordinates)
	 * 
	 * @param points  the input points
	 */
    public static void draw2DBoundingBox(Fenetre f, double[] box) {
    	int dim=2;
    	Point_2 p00=new Point_2(box[0], box[1]);
    	Point_2 p10=new Point_2(box[dim], box[1]);
    	Point_2 p01=new Point_2(box[0], box[dim+1]);
    	Point_2 p11=new Point_2(box[dim], box[dim+1]);
    	f.addSegment(p00, p10);
    	f.addSegment(p00, p01);
    	f.addSegment(p10, p11);
    	f.addSegment(p01, p11);
    }

	/**
	 * Draw a point cloud in a 3D frame (with no colors)
	 * 
	 * @param N the input points
	 */
    public static void draw3D(LinkedList<Point_3> N) {
    	System.out.print("Drawing point cloud in 3D...");
    	if(N==null) {
    		System.out.println("No point to draw");
    		return;
    	}
    	
    	LinkedList<Point_3> points3D=new LinkedList<Point_3>();
    	for(Point_3 p: N) {
    		Point_3 newPoint=(Point_3)p;
    		if(p.dimension()==3)
    			newPoint=new Point_3(p);
    		else
    			newPoint=new Point_3(p.getCartesian(0).doubleValue(), p.getCartesian(1).doubleValue(), 0.);
    		points3D.add(newPoint);
    	}
    	
		PointCloudViewer.points=points3D;
		PApplet.main(new String[] { "PointCloudViewer" }); // launch the 3D viewer. Warning: the input parameter must coincide with the name of the class (including package names)
    }

	/**
	 * Draw a point cloud in a 3D frame (with no colors)
	 * 
	 * @param N the input points
	 */
    /*public static void draw3D(PointCloud_KdTree N) {
    	int sizeCloud=PointCloud_KdTree.size(N);
    	Color[] colors=new Color[sizeCloud];
    	
    	PointCloud_KdTree t=N;
    	int i=0;
    	while(t!=null) {
    		colors[i]=Color.red; // every point has color red
    		t=t.next;
    		i++;
    	}
    	
    	draw3D(N, colors);
    }*/
    
	/**
	 * Draw a point cloud in 3D with colors.
	 * Points in the same cluster have the same color (colors are assigned at random). 
	 * 
	 * @param N the input points
	 * @param cluster defines how points are partitioned into clusters
	 */
    /*public static void draw3D(PointCloud_KdTree N, Cluster<Point_d> cluster) {
    	int sizeCloud=PointCloud_KdTree.size(N);
    	Color[] colors=new Color[sizeCloud]; // one color for each point in the cloud
    	Color[] randomColors=new Color[sizeCloud]; // one color for each cluster
    	
    	computeRandomColors(sizeCloud, randomColors); // compute colors at random
    	
    	PointCloud_KdTree t=N;
    	int i=0;
    	while(t!=null) { // assign a color to each point
    		if(cluster.getClusterIndex(t.p)==-1) // point t.p is not classified
    			colors[i]=Color.red;
    		else
    			colors[i]=randomColors[cluster.getClusterIndex(t.p)];
    		t=t.next;
    		i++;
    	}
    	
    	draw3D(N, colors);
    }*/

    /*public static void draw3D(PointCloud_KdTree N, Color[] colors) {
    	//System.out.println("Drawing point cloud in 3D...");
    	if(N==null || colors==null) {
    		System.out.println("No points (or colors) to draw");
    		return;
    	}
    	if(N.p.dimension()<3) {
    		System.out.println("Warning: drawing in dimension 3 a cloud of points of dimension "+N.p.dimension());
    		//return;
    	}
    	
    	List<Point_3> points3D=new ArrayList<Point_3>();
    	PointCloud_KdTree t=N;
    	while(t!=null) {
    		Point_3 newPoint=null;
    		if(N.p.dimension()==3)
    			newPoint=new Point_3(t.p);
    		else
    			newPoint=new Point_3(t.p.getCartesian(0).doubleValue(), t.p.getCartesian(1).doubleValue(), 0.);
    		points3D.add(newPoint);
    		t=t.next;
    	}
    	if(colors==null || points3D.size()!=colors.length) {
    		System.out.println("error: colors null");
    		return;
    	}
    	new MeshViewer(points3D, colors);
    	//System.out.println("3D drawing done");
    }*/
    
    private static void computeRandomColors (int n, Color[] cols) {
	// In this basic version, assign random colors
	for (int i=0; i<n; i++)
	    cols[i] = new Color (
	    			(float)Math.random(), 
	    			(float)Math.random(), 
	    			(float)Math.random());
    }

}
