import java.util.Calendar;
import java.util.LinkedList;

import Jcg.geometry.Point_;

/**
 * Class providing fast range search (using Kd-trees)
 * 
 * @author Luca Castelli Aleardi and Steve Oudot (Ecole Polytechnique, INF562)
 * @version jan 2020
 */
public class FastRangeSearch <X extends Point_> implements RangeSearch<X> {

	/** Kd-tree data structure storing the point cloud */
	private KdTree<X> kdTree=null;

	static long queryTime=0;
	static long constructionTime=0;
	static long basicQueryTime=0;    

	public static void timePerformance() {
		System.out.println("Range Search performaces:\nTotal construction time: " + constructionTime/1000 + "s " + 
				constructionTime%1000 + "ms");   	
		System.out.println("Total NN query time: " + queryTime/1000 + "s " + 
				queryTime%1000 + "ms\n");   	
	}

	/**
	 * Construct the data structure for range search
	 * 
	 * @param PointCloud_KdTree the input points
	 */
	public FastRangeSearch (LinkedList<X> N) {
		if (N == null || N.size()==0)
			kdTree = null;
		else {
			Calendar rightNow = Calendar.getInstance();
			long time0 = rightNow.getTimeInMillis();
			
			kdTree=new KdTree<X>(N); // construct the Kd-tree (recursive construction)

			rightNow = Calendar.getInstance();
			constructionTime = constructionTime+(rightNow.getTimeInMillis()-time0);
		}
	}


	/**
	 * Main search procedure (for Range search queries)
	 */
	public LinkedList<X> NN (X q, double sqRad) {
		Calendar rightNow = Calendar.getInstance(); // to compute query time
		long time0 = rightNow.getTimeInMillis();

		LinkedList<X> result=new LinkedList<X>();
		//choose your favorite Nearest neighbor search method
		//result=basicNN (q, N, sqRad);
		result=this.OrthogonalRangeSearch (q, sqRad); // efficient NN search

		rightNow = Calendar.getInstance(); // to compute query time
		queryTime = queryTime+(rightNow.getTimeInMillis()-time0);

		return result;
	}

	/**
	 * Perform range search using KdTrees
	 * 
	 * @param q the query point
	 * @param sqRad the (squared) radius of the Euclidean ball
	 * @return PointCloud the points in the range (epsilon close to q)
	 */
	public LinkedList<X> OrthogonalRangeSearch(X q, double sqRad) {
		if (kdTree == null)
			return null;
		else {
			LinkedList<X> result=new LinkedList<X>();
			result=kdTree.OrthogonalRangeSearch (q, sqRad);
			return result;
		}
	}
	
	public String toString() {
		return "Fast Kd-Tree search";
	}
	
}


