import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.*;
import java.util.LinkedList;

import javax.imageio.ImageIO;

import Jcg.geometry.Point_;
import Jcg.geometry.Point_2;
import Jcg.geometry.Point_3;
import Jcg.geometry.Point_d;

/**
 * Class providing methods for reading data (point clouds, images, ...) from file
 * 
 * @author Luca Castelli Aleardi and Steve Oudot(Ecole Polytechnique, INF562)
 * @version jan 2014
 */
public class IO {

    /**
     * Load point clouds from input file
     * 
     * @param filename the input file (text file storing point coordinates)
     * @return the list of input points
     */
    public static LinkedList pointsFromDataFile (String fileName) {
    	System.out.print("Loading point cloud from file "+fileName+" ...");
    	LinkedList<Point_> points = null;
    	try { 
    		BufferedReader in = new BufferedReader(new FileReader(fileName));
    		String s = null;
    		
    		int dim = -1;
    		
    		// Get dimension of point cloud
    		s = in.readLine();
    		if (s == null) {
    			in.close();
    			return points;
    		}
    		
    		// pass comments in data file
    		while (s.charAt(0) == '/')
    			s = in.readLine();
    		
    		int i = 0;
    		while ( s.charAt(i) == ' ' )
    			++i;
    		// i points to first digit of number
    		dim = (int)Double.parseDouble (s.substring (i));
    		
    		points = new LinkedList<Point_>();
    		
    		// Get the points, one after the other
    		double[] coords = new double [dim]; 
    		while ( (s = in.readLine()) != null ) {
    			i=0;
    			for (int d=0; d<dim-1; d++) {
    				while ( s.charAt(i) == ' ' )
    					++i;
    				// i points to first digit of coordinate
    				int j = i+1;
    				while ( s.charAt(j) != ' ')
    					++j;
    				// j points to space after coordinate
    				coords[d] = Double.parseDouble(s.substring(i, j));
    				i = j+1;
    			}
    			// Get last coordinate
    			while ( s.charAt(i) == ' ' )
    				++i;
    			// i points to first digit of last coordinate
    			coords[dim-1] = Double.parseDouble(s.substring(i));
    			// Append new point to point cloud
    			
    			Point_ p=null;
    			if(dim==2) p=new Point_2(coords[0], coords[1]);
    			else if(dim==3) p=new Point_3(coords[0], coords[1], coords[2]);
    			else p=new Point_d (coords);
    			
    			points.addFirst(p);
    			}
    			in.close();
    		} 	catch (Exception e) {  
    				e.printStackTrace();  
    			}
    	System.out.println(" done");
    	return points;
    }

}
