import java.util.LinkedList;

import Jcg.geometry.Point_;

/**
 * A class implementing a Kd-tree data structure
 * 
 * @author Luca Castelli Aleardi and Steve Oudot (Ecole Polytechnique, INF562)
 * @version jan 2020 - jan 2024
 */
public class KdTree<X extends Point_> {
	
	/** root node of the KdTree */
	public KdTreeNode<X> root=null;
	
	public static final int dirX=0, dirY=1, dirZ=2;
	
	/**
	 * Initialize the Kd-Tree
	 */    
	public KdTree(LinkedList<X> points) {
		if(points==null || points.size()==0)
			return; // the KdTree cannot be constructed (the input points are not defined)
		
		int dim=points.getFirst().dimension(); // dimension of the ambient space
		int cutDirection=dirX; // set the direction of the first cut: by default the cut direction is orthogonal to the X axis
		this.root=new KdTreeNode<X>(points, dim, cutDirection);
	}

	/**
	 * Range search: return the list of nearest point to a given query point q.
	 * The output is the set of points at distance at most sqRad from q.
	 */    
	public LinkedList<X> OrthogonalRangeSearch(X q, double sqRad) {
		if(this.root==null)
			return null;
		
		return this.root.OrthogonalRangeSearch(q, sqRad);
	}

	/**
	 * Return the list of points stored in the Kd-tree (computed recursively)
	 */    
	public LinkedList<X> getPoints() {
		LinkedList<X> points=new LinkedList<X>();
		
		if(this.root==null)
			return points;
		
		KdTreeNode.getPoints(this.root, points);
		return points;
	}

}
