import Jcg.geometry.*;
import Jcg.util.DLinkedList;

import java.util.*;;

/**
 * A class representing a node of a Kd-tree
 * 
 * @author Luca Castelli Aleardi and Steve Oudot (Ecole Polytechnique, INF562)
 * @version jan 2020-2024
 */
public class KdTreeNode<X extends Point_> {
	/** dimension of the input points */
	private int pointDimension; 
	/** direction of the cut */
	public int cutDimension;
	/** cut location: defining the coordinate of the cutting hyperplane (parallel to one of the axis) */
	public double cutValue;
	/** two descendant sub-trees */
	public KdTreeNode<X> lowerHalf, upperHalf;
	
	/** number of points stored in the rectangle box associated to this node */
	public int numPoints;
	/** point stored in a leaf (not defined for inner nodes) */
	public X point=null;
	
	// Below: additional information (redundant, not memory efficient)
	
	/** store the points associated to an internal node: to speed up the computation */
	private LinkedList<X> points; // not memory efficient

	/**
	 * Constructor -- builds the entire Kd-tree at once recursively
	 * 
	 * @param N 	 	list of points defining the point cloud (associated to the node)
	 * @param pDim 	 	dimension of the input points (typically 2 or 3)
	 * @param cutDim	cut dimension: defining the direction of the cutting hyperplane
	 */    
	public KdTreeNode(LinkedList<X> N, int pDim, int cutDim) {
		// TO BE COMPLETED
		throw new Error("To be completed: exercise 1.2");
	} 

	/**
	 * Range search: return the list of nearest point to a given query point q.
	 * The output is the set of points at distance at most sqRad from q.
	 * 
	 * @param q  		the query point
	 * @param sqRad 	squared radius
	 */    
	public LinkedList<X> OrthogonalRangeSearch(X q, double sqRad) {
		//COMPLETE
		throw new Error("To be completed: exercise 1.3");
	}

	//-----------------------------------
	//--- Auxiliary methods on trees ----
	//-----------------------------------

	/**
	 * Compute the size of a Kd-tree
	 * 
	 * @param t  		a node of the Kd-tree
	 */    
	public static<X extends Point_> int size(KdTreeNode<X> t) {
		int result=0;
		if(t==null) return 0;
		else {
			result=size(t.lowerHalf);
			result=result+size(t.upperHalf);
		}
		return result+1;
	}
	
	/**
	 * Check whether the tree is balanced
	 * 
	 * @param t  		a node of the Kd-tree
	 */    
	public static<X extends Point_> boolean checkBalance(KdTreeNode<X> t) {
		if (t.lowerHalf==null && t.upperHalf==null)
			return true;
		if (t.numPoints < 200) 
			return true;
		boolean result=true;
		if ((double)t.lowerHalf.numPoints/(double)t.numPoints < 0.25) {
			System.out.println("Bad balance in kd-tree! (" + 
					t.lowerHalf.numPoints + "/" + t.numPoints +
					"=" + (double)t.lowerHalf.numPoints/(double)t.numPoints + ")");
			result=false;
		}
		if ((double)t.lowerHalf.numPoints/(double)t.numPoints > 0.75) {
			System.out.println("Bad balance in kd-tree! (" + 
					t.lowerHalf.numPoints + "/" + t.numPoints +
					"=" + (double)t.lowerHalf.numPoints/(double)t.numPoints + ")");
			result=false;
		}
		result=result && checkBalance(t.lowerHalf);
		result=result && checkBalance(t.upperHalf);

		return result;
	}
	
	/**
	 * Compute the number of leaves in the tree
	 * 
	 * @param t  		a node of the Kd-tree
	 */    
	public static<X extends Point_> int leavesNumber(KdTreeNode<X> t) {
		int result=0;
		if(t==null) return 0;
		if(t.upperHalf==null && t.lowerHalf==null)
			return 1;
		else {
			result=leavesNumber(t.lowerHalf);
			result=result+leavesNumber(t.upperHalf);
		}
		return result;
	}
	
	/**
	 * Return a String representing a tree
	 * 
	 * @param t  		a node of the Kd-tree
	 */    
	public static<X extends Point_> String toString(KdTreeNode<X> t) {
		String result="";
		if(t==null) return result;
		if(t.upperHalf==null && t.lowerHalf==null)
			return ""+t.point+"\n";
		else {
			result=toString(t.lowerHalf);
			result=result+toString(t.upperHalf);
		}
		return result;
	}

	public static<X extends Point_> void getPoints(KdTreeNode<X> t, LinkedList<X> list) {
		if(t==null) return;
		if(t.upperHalf==null && t.lowerHalf==null)
			list.add(t.point);
		else {
			getPoints(t.lowerHalf, list);
			getPoints(t.upperHalf, list);
		}
	}

}
