import java.util.Arrays;
import java.util.LinkedList;

import Jcg.geometry.Point_;

/**
 * Computation of the median (via array sorting)
 * 
 * @author Luca Castelli Aleardi (Ecole Polytechnique, INF562)
 * @version jan 2014
 */
public class MedianWithSorting<X extends Point_> implements Median<X> {

	private final LinkedList<X> N; // input point set (warning: original points should not be sorted)
	
	public MedianWithSorting(LinkedList<X> points) {
		this.N=points;
	}

	/**
	 * Compute the median of a set of point cloud (after sorting)
	 * cutDim = {0,1,2} => {x,y,z}
	 * 
	 * @param cutDim indicates which are the coordinates to compare
	 * 
	 * Warning: input points should not be sorted (perform a copy of input points)
	 */    
	public X findMedian(int cutDim){
		if(this.N==null) return null;
		
		int size = this.N.size();
		if(size==0) return null;
		if(size==1) return N.getFirst();
		
		int index;
		if(size%2==1)
			index=size/2;
		else
			index=size/2-1;
		
		Point_[] buffer =new Point_[size]; // convert to a buffer array
		N.toArray(buffer);
		Arrays.sort(buffer, new CoordinateComparator<Point_>(cutDim)); // sort the temporary array
		return (X)buffer[index]; // return the middle point
	}

}
