import processing.core.*;

import java.io.File;
import java.util.LinkedList;

import Jcg.geometry.*;

/**
 * A simple 3d viewer for visualizing point clouds (based on Processing). It deals with methods for<p>
 * -) normals estimation <p>
 * -) outliers computation<p>
 * 
 * @author Luca Castelli Aleardi (INF555, 2016)
 *
 */
public class PointCloudViewer extends PApplet {

	public static LinkedList<Point_3> points; // input point cloud
	public static LinkedList<Point_3> neighbors=null; // closest neighbors
	public double radius=1.;
	
	int renderType=0; // choice of type of rendering
	int renderModes=3; // number of rendering modes
	DrawPointSet drawPointSet;
	
	/** For performing fast proximity queries with Kd-trees */
	public RangeSearch<Point_3> fastSearch;
	/** For performing slow proximity queries (brute-force quadratic algorithm) */
	public RangeSearch<Point_3> slowSearch;
	
	public static String filename;// input point clouds
	
	public void setup() {
		  size(1000,800,P3D);
		  ArcBall arcball = new ArcBall(this);
		  this.drawPointSet=new DrawPointSet(this, this.points, null);
		  
		  this.fastSearch=new FastRangeSearch<Point_3>(points);
		  slowSearch=new SlowRangeSearch<Point_3>(points);
	}
		 
		public void draw() {
		  background(200);
		  //this.lights();
		  /*directionalLight(101, 204, 255, -1, 0, 0);
		  directionalLight(51, 102, 126, 0, -1, 0);
		  directionalLight(51, 102, 126, 0, 0, -1);
		  directionalLight(102, 50, 126, 1, 0, 0);
		  directionalLight(51, 50, 102, 0, 1, 0);
		  directionalLight(51, 50, 102, 0, 0, 1);*/
		 
		  translate(width/2.f,height/2.f,-1*height/2.f);
		  this.strokeWeight(1);
		  stroke(150,150,150);
		  
		  this.drawPointSet.draw(renderType);
		  this.drawOptions();
		}
		
		public void keyPressed(){
			  switch(key) {
			    case('+'): this.zoomIn(); break;
			    case('-'): this.zoomOut(); break;
			    case('i'):case('I'): this.updateRadius(1.6); break;
			    case('d'):case('D'): this.updateRadius(0.6); break;
			    case('f'):case('F'): this.fastRangeSearch(); break;
			    case('r'):this.renderType=(this.renderType+1)%this.renderModes; break;
			    case('s'):case('S'): this.slowRangeSearch(); break;
			  }
		}
		
		public void drawOptions() {
			int hF=12;
			fill(255);
			this.textMode(this.SCREEN);
			this.text("'+' or '-' for zooming", 10, hF*3);
			this.text("'i' or 'd' for increasing or decreasing the radius", 10, hF*4);
			this.text("'s' for slow range search", 10, hF);
			this.text("'f' for fast range search", 10, hF*2);
		}

		public void zoomIn() {
			this.drawPointSet.zoom=this.drawPointSet.zoom*1.5;
		}

		public void zoomOut() {
			this.drawPointSet.zoom=this.drawPointSet.zoom*0.75;
		}

		public void scaleNormal(double scale) {
			this.drawPointSet.normalScale*=scale;
		}

		public void updateRadius(double scale) {
			this.radius*=scale;
			System.out.println("Radius Updated: "+this.radius);
		}

		public void fastRangeSearch() {
			int n=this.points.size();	
			int index=(int)(Math.random()*n);
			Point_3 q=this.points.get(index); // query point

			this.neighbors=this.fastSearch.OrthogonalRangeSearch(q, this.radius);
			this.drawPointSet.points2=neighbors;
			System.out.println("Closest neighbors at distance r<="+radius+": "+this.neighbors.size());
		}

		public void slowRangeSearch() {
			int n=this.points.size();	
			int index=(int)(Math.random()*n);
			Point_3 q=this.points.get(index); // query point

			this.neighbors=this.slowSearch.OrthogonalRangeSearch(q, this.radius);
			this.drawPointSet.points2=neighbors;
			System.out.println("Closest neighbors at distance r<="+radius+": "+this.neighbors.size());
		}

		/**
		 * For running the PApplet as Java application
		 */
		public static void main(String args[]) {
			if(args.length!=1) {
				System.out.println("Wrong number of input parameters: required one file .off");
				System.exit(0);
			}
			
			PApplet.main(new String[] { "PointCloudViewer" });
		}

}
