import java.awt.image.BufferedImage;
import java.util.LinkedList;

import Jcg.geometry.Point_;
import Jcg.geometry.Point_2;
import Jcg.geometry.Point_d;
import Jcg.viewer.old.Fenetre;

/**
 * This class contains methods for testing the Kd-tree data structure
 * 
 * @author Luca Castelli Aleardi (Ecole Polytechnique, INF562)
 * @version jan 2014
 */
public class TestMedian {

	public static void main(String[ ] args) throws Exception {
		System.out.println("Exercice 1.1: testing median computation");
		LinkedList points=null; // input point cloud
		int dimension=0; // dimension not defined yet: either 2 or 3

		if (args.length < 1) {
			points=PointSetUtil.randomPointsOnAnnulus(100, 1.0);
		}
		else {
			String filename=args[0]; // input dataset: either point cloud (.dat) or image file (.jpg, .png, .bmp)

			if(filename.endsWith(".dat")) {
				System.out.println("Loading point cloud from data: "+filename);
				points=IO.pointsFromDataFile(filename);
			}
			else {
				throw new Error("Error: wrong input format");
			}
		}

		dimension=((Point_)points.getFirst()).dimension();

		testMedian(points, 0); // compute median by performing a cut along x-direction
		testMedian(points, 1); // compute median by performing a cut along y-direction
		if(dimension==3)
			testMedian(points, 2); // compute median by performing a cut along z-direction

		System.out.println("Test done");
	}

	public static<X extends Point_> void testMedian(LinkedList<X> N, int cutDirection) {
		if(N==null)
			return;

		Median<X> median; // choice of the median computation
		//median=new LinearTimeMedian(N);
		median=new MedianWithSorting<X>(N);
		
		X p=median.findMedian(cutDirection); // compute median point
		double value=p.getCartesian(cutDirection).doubleValue(); // cut value
		
		System.out.print("Splitting point set along direction "+cutDirection+"...");
		LinkedList<X>[] subSets=PointSetUtil.split(N, value, cutDirection); // split input points
		System.out.println("done");
		
		System.out.println("\tsize of the original point set: "+N.size());
		
		System.out.println("\tsize lower half: "+subSets[0].size());
		System.out.println("\tsize upper half: "+subSets[1].size());
		System.out.println("\tcut value: "+value);

		/*
		if(N.p.dimension()==2) {
			Draw.draw2D(subN[0], "lower points");
			Draw.draw2D(subN[1], "upper points");
		}*/
	}

}
